"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.setupApp = void 0;
const common_1 = require("@nestjs/common");
const swagger_1 = require("@nestjs/swagger");
const core_1 = require("@nestjs/core");
const morgan_1 = __importDefault(require("morgan"));
const core_2 = require("./modules/core");
const setupApp = async (app) => {
    const bootstrapOptions = app.get(core_2.BOOTSTRAP_OPTIONS_TOKEN);
    const { swagger, version } = bootstrapOptions;
    // version must be loaded before swagger
    app.enableVersioning({
        type: common_1.VersioningType.URI,
    });
    const config = new swagger_1.DocumentBuilder()
        .setTitle(swagger?.title ?? '')
        .setDescription(swagger?.description ?? '')
        .setVersion(version)
        .build();
    const document = swagger_1.SwaggerModule.createDocument(app, config);
    swagger_1.SwaggerModule.setup('doc', app, document);
    const validationPipe = app.get(core_2.ValidationPipe);
    const contextInterceptor = app.get(core_2.ContextInterceptor);
    app.useGlobalPipes(validationPipe);
    app.useGlobalInterceptors(contextInterceptor);
    const adapterHost = app.get(core_1.HttpAdapterHost);
    const logger = app.get(core_2.RootLogger);
    app.useGlobalFilters(new core_2.AllExceptionsFilter(adapterHost, logger));
    app.use((0, morgan_1.default)((tokens, req, res) => [
        tokens.method(req, res),
        tokens.url(req, res),
        tokens.status(req, res),
        tokens.res(req, res, 'content-length'),
        '-',
        tokens['response-time'](req, res),
        'ms',
        '- TraceId: ',
        tokens.res(req, res, core_2.TRACE_ID),
    ].join(' '), {
        stream: logger.stream,
        skip(req) {
            return req.url === '/health';
        },
    }));
    app.enableShutdownHooks();
    return app;
};
exports.setupApp = setupApp;
//# sourceMappingURL=app.js.map