"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequiredDomainClass = exports.NonRequiredDomainClass = void 0;
const model_properties_accessor_1 = require("@nestjs/swagger/dist/services/model-properties-accessor");
const common_1 = require("@nestjs/common");
const mapped_types_1 = require("@nestjs/mapped-types");
const class_validator_1 = require("class-validator");
const constants_1 = require("@nestjs/swagger/dist/constants");
const plugin_constants_1 = require("@nestjs/swagger/dist/plugin/plugin-constants");
const api_property_decorator_1 = require("@nestjs/swagger/dist/decorators/api-property.decorator");
const validator_1 = require("./validator");
const modelPropertiesAccessor = new model_properties_accessor_1.ModelPropertiesAccessor();
/**
 * Adapted from: https://github.com/nestjs/swagger/blob/master/lib/type-helpers/partial-type.helper.ts
 *
 * Note: this class relies internal implementations of the @nestjs/swagger package.
 *
 * Reason for class:
 * `PartialType` from @nestjs/swagger takes a class and makes all properties optional. In this case, optional means
 * either `<a value>`, `null` or `undefined`.
 *
 * However, there are use cases where we want optional to mean a value or `<a value>` or `undefined` but not `null`.
 * An example usecase is optionally updating a non-nullable property. Normally, `null` means delete the value.
 *
 * So this class, takes required properties and allows them to be undefined or a value, and takes non required properties and makes them completely optional
 *
 * @param classRef
 * @param overrides
 * @constructor
 */
function TypeBuilder(classRef, overrides) {
    const fields = modelPropertiesAccessor.getModelProperties(classRef.prototype);
    class ModifiedTypeClass {
        constructor() {
            (0, mapped_types_1.inheritPropertyInitializers)(this, classRef);
        }
    }
    function applyDecorator(key, metadata) {
        const config = overrides != null ? overrides[key] : undefined;
        const required = config?.required ?? metadata?.required ?? true;
        const nullable = metadata?.nullable ?? true;
        const decorators = [
            (0, api_property_decorator_1.ApiProperty)({
                ...metadata,
                required,
                nullable,
            }),
        ];
        if (nullable) {
            decorators.push((0, class_validator_1.IsOptional)());
        }
        else {
            decorators.push((0, validator_1.IsNotNullAndOptional)());
        }
        const decorator = (0, common_1.applyDecorators)(...decorators);
        decorator(ModifiedTypeClass.prototype, key);
    }
    (0, mapped_types_1.inheritValidationMetadata)(classRef, ModifiedTypeClass);
    (0, mapped_types_1.inheritTransformationMetadata)(classRef, ModifiedTypeClass);
    fields.forEach((key) => {
        const metadata = Reflect.getMetadata(constants_1.DECORATORS.API_MODEL_PROPERTIES, classRef.prototype, key) || {};
        applyDecorator(key, metadata);
    });
    if (ModifiedTypeClass[plugin_constants_1.METADATA_FACTORY_NAME]) {
        const pluginFields = Object.keys(ModifiedTypeClass[plugin_constants_1.METADATA_FACTORY_NAME]());
        pluginFields.forEach((key) => applyDecorator(key));
    }
    return ModifiedTypeClass;
}
const NonRequiredDomainClass = (classRef) => {
    const fields = modelPropertiesAccessor.getModelProperties(classRef.prototype);
    const fieldConfigs = fields.reduce((agg, key) => {
        const metadata = Reflect.getMetadata(constants_1.DECORATORS.API_MODEL_PROPERTIES, classRef.prototype, key) || {};
        const fieldConfig = {
            nullable: metadata?.nullable,
            required: false,
        };
        return {
            ...agg,
            [key]: fieldConfig,
        };
    }, {});
    return TypeBuilder(classRef, fieldConfigs);
};
exports.NonRequiredDomainClass = NonRequiredDomainClass;
const RequiredDomainClass = (classRef) => {
    const fields = modelPropertiesAccessor.getModelProperties(classRef.prototype);
    const fieldConfigs = fields.reduce((agg, key) => {
        const metadata = Reflect.getMetadata(constants_1.DECORATORS.API_MODEL_PROPERTIES, classRef.prototype, key) || {};
        const fieldConfig = {
            nullable: metadata.nullable || metadata.default != null,
            required: true,
        };
        return {
            ...agg,
            [key]: fieldConfig,
        };
    }, {});
    return TypeBuilder(classRef, fieldConfigs);
};
exports.RequiredDomainClass = RequiredDomainClass;
//# sourceMappingURL=typeBuilder.js.map