"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IsValidDate = exports.IsDefined = exports.IsNotNullAndOptional = exports.IsNotNull = exports.Min = exports.Max = void 0;
const class_validator_1 = require("class-validator");
const dayjs_1 = __importDefault(require("dayjs"));
const customParseFormat_1 = __importDefault(require("dayjs/plugin/customParseFormat"));
const common_1 = require("@nestjs/common");
dayjs_1.default.extend(customParseFormat_1.default);
const parseNumber = (value) => {
    let number;
    if (typeof value === 'string') {
        number = parseFloat(value);
    }
    else {
        number = value;
    }
    return number;
};
const Max = (max) => (0, class_validator_1.ValidateBy)({
    name: class_validator_1.MAX,
    constraints: [max],
    validator: {
        validate(value) {
            const number = parseNumber(value);
            if (Number.isNaN(number)) {
                return false;
            }
            return number <= max;
        },
        defaultMessage() {
            return '$property must not be greater than $constraint1';
        },
    },
});
exports.Max = Max;
const Min = (min) => (0, class_validator_1.ValidateBy)({
    name: class_validator_1.MIN,
    constraints: [min],
    validator: {
        validate(value) {
            const number = parseNumber(value);
            if (Number.isNaN(number)) {
                return false;
            }
            return number >= min;
        },
        defaultMessage() {
            return '$property must not be less than $constraint1';
        },
    },
});
exports.Min = Min;
const IsNotNull = () => (0, common_1.applyDecorators)((0, class_validator_1.IsNotIn)([null], {
    message: '$property must not be null',
}));
exports.IsNotNull = IsNotNull;
const IsNotNullAndOptional = () => (0, common_1.applyDecorators)((0, class_validator_1.ValidateIf)((object, value) => value !== undefined), (0, exports.IsNotNull)());
exports.IsNotNullAndOptional = IsNotNullAndOptional;
const IsDefined = () => (0, common_1.applyDecorators)((0, class_validator_1.IsNotIn)([undefined], {
    message: '$property must be defined',
}));
exports.IsDefined = IsDefined;
const IsValidDate = (format = 'YYYY-MM-DD[T]HH:mm:ss[Z]') => (0, class_validator_1.ValidateBy)({
    name: class_validator_1.IS_DATE,
    validator: {
        validate(value) {
            if (value instanceof Date) {
                return !Number.isNaN(value.getTime());
            }
            if (typeof value !== 'string') {
                return false;
            }
            return (0, dayjs_1.default)(value, format, true).isValid();
        },
        defaultMessage() {
            return `$property must be a valid date of the format ${format}`;
        },
    },
});
exports.IsValidDate = IsValidDate;
//# sourceMappingURL=validator.js.map