"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.logger = void 0;
const winston_1 = require("winston");
const { combine, timestamp, colorize, json } = winston_1.format;
const loggerFormat = (0, winston_1.format)((info) => {
    const traceId = info.traceId ? ` TraceId: ${info.traceId}` : '';
    return {
        ...info,
        message: `${info.level.toUpperCase()}: ${info.timestamp} ${info.message}${traceId}`,
    };
});
let customFormat = combine(timestamp(), loggerFormat(), json());
if (process.env.ENABLE_COLORIZED_LOG === 'true') {
    customFormat = combine(timestamp(), loggerFormat(), colorize({ all: true }), json());
}
exports.logger = (0, winston_1.createLogger)({
    format: customFormat,
    transports: [
        new winston_1.transports.Console({
            level: process.env.LOG_LEVEL || 'info',
            handleExceptions: true,
        }),
    ],
});
//# sourceMappingURL=logger.js.map